function [LocalVol Q pu pm pd] = DKCTrinomial3(call,put,Spot,T,r,v,n)

% Calculate the time increment (dt), the moves up (u) and down (d).
dt = T/n;
u = exp(v*sqrt(2*dt));
d = 1/u;

% New tress
S = zeros(2*n+1,n+1);
S(n+1,1) = Spot;
for j = 2:n+1
    for i = n-j+2:n+j
        S(i,j) = Spot*u^(n+1)*d^i;
	end
end

F = zeros(2*n+1,n+1);
F(n+1,1) = Spot*exp(r*dt);
for j = 2:n+1
    for i = n-j+2:n+j
        F(i,j) = S(i,j)*exp(r*dt);
	end
end

% Initialize the matrices for Arrow-Debreu prices (AD),
% up probability (p), and down probability (q)
Q = zeros(2*n+1,n+1);
p = zeros(2*n+1,n+1);
q = zeros(2*n+1,n+1);


% Initialize the first node.
Q(n+1,1) = 1;
q(n+1,1) = exp(r*dt)*put(n+1,1)/ Q(n+1,1)/(S(n+1,2)-S(n+2,2));
p(n+1,1) = (F(n+1,1)+q(n+1,1)*(S(n+1,2)-S(n+2,2))-S(n+1,2))/(S(n,2)-S(n+1,2));
Q(n,2) = p(n+1,1)* Q(n+1,1)*exp(-r*dt);
Q(n+1,2) = (1-p(n+1,1)-q(n+1,1))* Q(n+1,1)*exp(-r*dt);
Q(n+2,2) = q(n+1,1)* Q(n+1,1)*exp(-r*dt);

for j=2:n
	% Nodes center and below
	for i=n+1:n+j
		E = 0;
		for k = i+1:n+j
			E = E + Q(k,j)*(S(i,j) - F(k,j));
		end
		q(i,j) = (exp(r*dt)*put(i,j)-E)/ Q(i,j)/(S(i,j+1) - S(i+1,j+1));
		p(i,j) = (F(i,j)+q(i,j)*(S(i,j+1)-S(i+1,j+1))-S(i,j+1))/(S(i-1,j+1)-S(i,j+1));
		if q(i,j) <= 0  | q(i,j) >= 1 | p(i,j) <= 0 | p(i,j) >= 1
			F1 = F(i,j);
			S0 = S(i-1,j+1);
			S1 = S(i  ,j+1);
			S2 = S(i+1,j+1);
			if S1 < F1 & F1 < S0
				p(i,j) = ((F1-S1)/(S0-S1)+(F1-S2)/(S0-S2))/2;
				q(i,j) = (S0-F1)/(S0-S2)/2;
			elseif S2 < F1 & F1 < S1
				q(i,j) = ((S0-F1)/(S0-S2)+(S1-F1)/(S1-S2))/2;
				p(i,j) = (F1-S2)/(S0-S2)/2;
			end
		end
	end

	% Nodes above
	for i=n-j+2:n
		E = 0;
		for k = n-j+2:i-1
			E = E - Q(k,j)*(S(i,j) - F(k,j));
		end
		p(i,j) = (exp(r*dt)*call(i,j)-E)/Q(i,j)/(S(i-1,j+1) - S(i,j+1));
		q(i,j) = (F(i,j)-p(i,j)*(S(i-1,j+1)-S(i,j+1))-S(i,j+1))/(S(i+1,j+1)-S(i,j+1));
		if q(i,j) <= 0 | q(i,j) >= 1 | p(i,j) <= 0 | p(i,j) >= 1
			F1 = F(i,j);
			S0 = S(i-1,j+1);
			S1 = S(i  ,j+1);
			S2 = S(i+1,j+1);
			if S1 < F1 & F1 < S0
				p(i,j) = ((F1-S1)/(S0-S1)+(F1-S2)/(S0-S2))/2;
				q(i,j) = (S0-F1)/(S0-S2)/2;
			elseif S2 < F1 & F1 < S1
				q(i,j) = ((S0-F1)/(S0-S2)+(S1-F1)/(S1-S2))/2;
				p(i,j) = (F1-S2)/(S0-S2)/2;
			end
		end
	end
	% Update Arrow-Debreu security prices
	for i = n-(j+1)+2:n+(j+1)
		if i == n-(j+1)+2  % Uper node
			Q(i,j+1) = p(i+1,j)*Q(i+1,j)*exp(-r*dt);
		elseif i == n-(j+1)+3 % Second node
			Q(i,j+1) = ((1-p(i,j)-q(i,j))*Q(i,j) + p(i+1,j)*Q(i+1,j))*exp(-r*dt);
		elseif i == n+(j+1)-1 % Second to last node
			Q(i,j+1) = ((1-p(i,j)-q(i,j))*Q(i,j) + q(i-1,j)*Q(i-1,j))*exp(-r*dt);
		elseif i == n+(j+1)   % Last node
			Q(i,j+1) = q(i-1,j)*Q(i-1,j)*exp(-r*dt);
		else             % Remaining nodes
			Q(i,j+1) = (q(i-1,j)*Q(i-1,j) + p(i+1,j)*Q(i+1,j) + (1-p(i,j)-q(i,j))*Q(i,j))*exp(-r*dt);
		end
	end
end

% Create Local Volatilities
LocalVol = zeros(2*n-1,n);
pu = zeros(2*n-1,n);
pd = zeros(2*n-1,n);
pm = zeros(2*n-1,n);


for j = 1:n
    for i = n-j+2:n+j
        pu(i,j) = p(i,j);
        pd(i,j) = q(i,j);
        pm(i,j) = 1-pu(i,j)-pd(i,j);
        F0 = (pu(i,j)*S(i-1,j+1) + ...
			  pm(i,j)*S(i  ,j+1) + ...
			  pd(i,j)*S(i+1,j+1)) * exp(r*dt);
        LocalVol(i,j) = sqrt((pu(i,j)*(S(i-1,j+1)-F0)^2 +...
			                  pm(i,j)*(S(i  ,j+1)-F0)^2 +...
							  pd(i,j)*(S(i+1,j+1)-F0)^2)/(F0^2*dt));
	end
end

% Rescale pu, pm, and pd, and LocalVol
pu = pu(2:end,:);
pd = pd(2:end,:);
pm = pm(2:end,:);
LocalVol = LocalVol(2:end,:);

% pu = pu(2:end-1,1:end-1);
% pm = pm(2:end-1,1:end-1);
% pd = pd(2:end-1,1:end-1);

% for j = 1:n
%     for i = n-j+1:n+j-1
%         pu(i,j) = p(i,j);
%         pd(i,j) = q(i,j);
%         pm(i,j) = 1-pu(i,j)-pd(i,j);
%         F0 = (pu(i,j)*S(i-1,j+1) + ...
% 			  pm(i,j)*S(i  ,j+1) + ...
% 			  pd(i,j)*S(i+1,j+1)) * exp(r*dt);
% 		[j i F0]
%         LocalVol(i,j) = sqrt((pu(i,j)*(S(i-1,j+1)-F0)^2 +...
% 			                  pm(i,j)*(S(i  ,j+1)-F0)^2 +...
% 							  pd(i,j)*(S(i+1,j+1)-F0)^2)/(F0^2*dt));
% 	end
% 	'--------------------------'
% end

% LocalVol = 99999;

% p = p(2:end-1,1:end-1);
% q = q(2:end-1,1:end-1);
% LocalVol = LocalVol(2:end-1,:);

